<?php
declare(strict_types=1);

/**
 * This class uses Carbon for advanced date handling.
 * (c) Brian Nesbitt <brian@nesbot.com>
 *
 * @link http://carbon.nesbot.com/
 *       For the full copyright and license information, please view the LICENSE
 *       file that was distributed with this source code.
 */

namespace Infomaker\Everyware\Support;

use Carbon\Carbon;
use DateTimeZone;
use Illuminate\Support\Traits\Macroable;
use IntlDateFormatter;
use JsonSerializable;

/**
 * Date
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Support
 * @since   Infomaker\Everyware\Support\Date 0.1
 */
class Date extends Carbon implements JsonSerializable
{

    use Macroable;

    /**
     * The format of a date in Open Content
     */
    public const OC = 'Y-m-d\TH:i:s\Z';

    /**
     * The custom Carbon JSON serializer.
     *
     * @var callable|null
     */
    protected static $serializer;

    /**
     * Contains an instance of the formatter used to translate dates into text
     *
     * @var IntlDateFormatter|null
     */
    protected static $formatter;

    /**
     * Contains the timezone set as an environment-variable on the server
     *
     * @var string
     */
    protected static $default_timezone;

    /**
     * Contains the local/language set as an environment-variable on the server
     *
     * @var string
     */
    protected static $default_locale;

    /**
     * Date constructor.
     *
     *
     * @param string|null              $time
     * @param DateTimeZone|string|null $tz
     */
    public function __construct($time = null, $tz = null)
    {
        static::$default_timezone = getenv('PHP_TIMEZONE') ?: 'Europe/Stockholm';
        static::$default_locale = getenv('PHP_DEFAULT_LOCALE') ?: 'sv-SE';

        static::setLocale(explode('_', Str::snake(static::$default_locale))[0]);
        date_default_timezone_set(static::$default_timezone);

        parent::__construct($time, $tz);
    }

    /**
     * Prepare the object for JSON serialization.
     *
     * @return array|string
     */
    public function jsonSerialize()
    {
        if (static::$serializer) {
            return \call_user_func(static::$serializer, $this);
        }
        $carbon = $this;

        return \call_user_func(function () use ($carbon) {
            return get_object_vars($carbon);
        });
    }

    /**
     * Convert date into a format readable to Open Content
     * To avoid resetting timezone on the original date we create a copy first
     * @return string
     */
    public function toOcString(): string
    {
        $copy = $this->copy()->setTimezone('UTC'); // Reset Timezone

        return $copy->format(static::OC);
    }

    /**
     * Use the custom formatter to format and translate a date into the requested format
     *
     * @param $format
     *
     * @uses  IntlDateFormatter::formatObject()
     * @since 0.1
     * @return string
     */
    public function toText($format): string
    {
        return static::formatter()->formatObject($this, $format);
    }

    /**
     * Create a date using the format from Open Content
     *
     * @param string $date
     *
     * @return static
     */
    public static function createFromOcString($date)
    {
        return static::createFromTimestamp(strtotime($date));
    }

    /**
     * JSON serialize all Carbon instances using the given callback.
     *
     * @param  callable $callback
     *
     * @return void
     */
    public static function serializeUsing($callback): void
    {
        static::$serializer = $callback;
    }

    /**
     * Function for retrieving the formatter used to format dates into human readable string
     *
     * @return IntlDateFormatter
     */
    public static function formatter(): IntlDateFormatter
    {
        if (static::$formatter === null) {
            static::setFormatter(static::$default_locale);
        }

        return static::$formatter;
    }

    /**
     * Function for retrieving the formatter used to format dates into human readable string
     *
     * @param string $local
     *
     * @return void
     */
    public static function setFormatter($local = ''): void
    {
        static::$formatter = new IntlDateFormatter ($local, // string locale
            IntlDateFormatter::FULL, // int date type
            IntlDateFormatter::FULL // int time type
        );
    }
}
