<?php

namespace Infomaker\Everyware\Support\Storage;


use Infomaker\Everyware\Support\Collection;

/**
 * CollectionDB
 *
 * @link    http://infomaker.se
 * @package Infomaker\ProjectPlugin\Storage
 * @since   Infomaker\ProjectPlugin\Storage\CollectionDB 0.1
 */
class CollectionDB
{

    /**
     * Contains the option name where the collection is stored
     *
     * @var string
     */
    protected $option_name;

    /**
     * Contains whether the collection is global
     *
     * @var bool
     */
    protected $global = false;

    /**
     * Contains the data collection
     *
     * @var Collection
     */
    private $collection;

    /**
     * CollectionDB constructor.
     *
     *
     * @param string $option_name
     * @param bool   $global
     */
    public function __construct($option_name, $global = false)
    {
        $this->option_name = $option_name;
        $this->global      = $global;
        $this->setCollection($this->getCollection());
    }

    /**
     * A way to set the collection
     *
     * @param mixed $collection
     *
     * @return CollectionDB
     */
    public function setCollection($collection = [])
    {
        $this->collection = new Collection($collection);

        return $this;
    }

    /**
     * Retrieve data from ObjectDB depending on global state and convert it to a collection before sending it back
     *
     * @return Collection
     */
    private function getCollection()
    {
        return $this->global ? ObjectDB::getGlobal(null, $this->option_name, []) : ObjectDB::get($this->option_name, []);
    }

    /**
     * Create a collection table with optional data
     *
     * @param string $option_name
     * @param bool   $global
     * @param array  $items
     *
     * @return CollectionDB
     */
    public static function create($option_name, $global = false, array $items = [])
    {
        $collection = new static($option_name, $global);
        $collection->setCollection($items)->save();

        return $collection;
    }

    /**
     * Save collection state to database
     *
     * @return bool
     */
    public function save()
    {
        return $this->global ? ObjectDB::updateGlobal(null, $this->option_name, $this->collection) : ObjectDB::update($this->option_name, $this->collection);
    }

    /**
     * Drop the collection table
     *
     * @param string $option_name
     * @param bool   $global
     *
     * @return bool
     */
    public static function drop($option_name, $global = false)
    {
        return $global ? ObjectDB::removeGlobal(null, $option_name) : ObjectDB::remove($option_name);
    }

    /**
     * Static function to connect to the options table and retrieve all data as collection
     *
     * @param string $option_name
     *
     * @return Collection
     */
    public static function table($option_name)
    {
        return (new static($option_name))->all();
    }

    /**
     * Retrieve the hole collection of data
     *
     * @return Collection
     */
    public function all()
    {
        return $this->collection;
    }

    /**
     * Static function to connect to the global options table and retrieve all data as collection
     *
     * @param string $option_name
     *
     * @return Collection
     */
    public static function GlobalTable($option_name)
    {
        return (new static($option_name, true))->all();
    }

    /**
     * Add an item to the collection
     *
     * @param mixed $item
     *
     * @return CollectionDB
     */
    public function add($item)
    {
        $this->collection->push($item);

        return $this;
    }

    /**
     * Merge items into the collection
     *
     * @param $items
     *
     * @return Collection
     */
    public function merge($items)
    {
        $this->setCollection($this->collection->merge($items));

        return $this->all();
    }

    /**
     * Remove an item from the collection
     *
     * @param $key
     *
     * @return CollectionDB
     */
    public function remove($key)
    {
        $this->collection->forget($key);

        return $this;
    }

    /**
     * Reset collection
     *
     * @return CollectionDB
     */
    public function reset()
    {
        return $this->setCollection();
    }

    /**
     * Update an existing item in the collection
     *
     * @param $key
     * @param $item
     *
     * @return CollectionDB
     */
    public function update($key, $item)
    {
        $this->collection->put($key, $item);

        return $this;
    }
}
